# -*- coding: utf-8 -*-
import json
from multiprocessing import Process, Queue
from util import LogUtils
from util import KafkaUtils
from loguru import logger
from concurrency.FileUpdateThread import ImageFileUpdate
from concurrency.HeartbeatThread import Heartbeat
import time


class CommonProcess(Process):
    def __init__(self, fbQueue, hbQueue, content, msg, imageQueue, mode_service):
        super().__init__()
        self.fbQueue = fbQueue
        self.hbQueue = hbQueue
        self.content = content
        self.msg = msg
        self.mode_service = mode_service
        self.imageQueue = imageQueue
        self.eventQueue = Queue()


    # 给本进程发送事件
    def sendEvent(self, eBody):
        self.eventQueue.put(eBody)

    # 获取下一个事件
    def getEvent(self):
        eBody = None
        try:
            eBody = self.eventQueue.get(block=False)
        except Exception as e:
            pass
        return eBody

    def getFeedback(self):
        eBody = None
        try:
            eBody = self.fbQueue.get(block=False)
        except Exception as e:
            pass
        return eBody

    def run(self):
        # 初始化日志配置
        LogUtils.init_log(self.content)
        logger.info("心跳、图片上传，反馈进程开始执行, requestId:{}", self.msg.get("request_id"))
        # 启动心跳线程
        hb = Heartbeat(self.fbQueue, self.hbQueue, self.msg.get("request_id"), self.mode_service)
        hb.start()
        # 图片上传线程
        imageFileUpdate = ImageFileUpdate(self.fbQueue, self.content, self.msg, self.imageQueue, self.mode_service)
        imageFileUpdate.start()
        kafkaProducer = KafkaUtils.CustomerKafkaProducer(self.content)
        # 心跳线程检测
        heartbeat_num = 0
        # 图片上传线程检测
        imageFileUpdate_num = 0
        while True:
            try:
                fb = self.getFeedback()
                if fb is not None and len(fb) > 0:
                    logger.info("发送结果反馈：{}", json.dumps(fb))
                    kafkaProducer.get_producer()
                    kafkaProducer.sender(self.content["kafka"]["topic"]["dsp-alg-results-topic"],
                                         fb["request_id"], fb, 1)
                if heartbeat_num == 0 and not hb.is_alive():
                    logger.error("未检测到心跳线程活动，心跳线程可能出现异常, reuqestId:{}", self.msg.get("request_id"))
                    break
                if imageFileUpdate_num == 0 and not imageFileUpdate.is_alive():
                    logger.error("未检测到图片上传线程活动，图片上传线程可能出现异常, reuqestId:{}", self.msg.get("request_id"))
                    break
                eBody = self.getEvent()
                if eBody is not None and len(eBody) > 0:
                    cmdStr = eBody.get("command", None)
                    # 接收心跳线程和图片上传停止指令
                    if 'stop_heartbeat_imageFileUpdate' == cmdStr:
                        heartbeat_num += 1
                        imageFileUpdate_num += 1
                        hb.run_status = False
                        imageFileUpdate.run_status = False
                        hb.join()
                        imageFileUpdate.join()
                        # 接收进程停止指令
                    if 'stop' == cmdStr:
                        hb.run_status = False
                        imageFileUpdate.run_status = False
                        hb.join()
                        imageFileUpdate.join()
                        break
            except Exception as e:
                logger.error("requestId:{}, 结果反馈异常：", self.msg.get("request_id"))
                logger.exception(e)
        logger.info("心跳、图片上传，反馈进程执行完成, requestId:{}", self.msg.get("request_id"))



