from threading import Thread
from loguru import logger
import cv2
from util.AliyunSdk import AliyunOssSdk
from util import TimeUtils
import uuid
from entity import FeedBack
from enums.AnalysisStatusEnum import AnalysisStatus
import numpy as np
from PIL import Image

class FileUpdate(Thread):
    def __init__(self, fbQueue, content, msg, imageQueue, mode_service):
        super().__init__()
        self.fbQueue = fbQueue
        self.content = content
        self.run_status = True
        self.imageQueue = imageQueue
        self.mode_service = mode_service
        self.msg = msg
        self.cfg = {}

    # 执行配置项
    def setCfg(self, key, value):
        self.cfg[key] = value

    # # 给本线程发送事件
    # def sendEvent(self, eBody):
    #     self.eventQueue.put(eBody)

    # # 获取下一个事件
    # def getEvent(self):
    #     return self.eventQueue.get()

    # # 获取下一个事件
    def getImageQueue(self):
        eBody = None
        try:
            eBody = self.imageQueue.get(block=False)
        except Exception as e:
            pass
        return eBody

    # 推送执行结果
    def sendResult(self, result):
        self.fbQueue.put(result)


class ImageFileUpdate(FileUpdate):

    def run(self):
        logger.info("开始启动图片上传线程, requestId:{}", self.msg.get("request_id"))
        aliyunOssSdk = AliyunOssSdk(self.content)
        aliyunOssSdk.get_oss_bucket()
        while self.run_status:
            try:
                image_dict = self.getImageQueue()
                if image_dict is not None and len(image_dict) > 0:
                    # 图片帧数编码
                    or_result, or_image = cv2.imencode(".jpg", image_dict.get("or_frame"))
                    ai_result, ai_image = cv2.imencode(".jpg", image_dict.get("ai_frame"))
                    # 定义上传图片名称
                    random_num = str(uuid.uuid1().hex)
                    time_now = TimeUtils.now_date_to_str("%Y-%m-%d-%H-%M-%S")
                    # 图片名称待后期修改
                    or_image_name = self.build_image_name(self.msg.get('results_base_dir'), time_now,
                                                         str(image_dict.get("current_frame")),
                                                         str(image_dict.get("last_frame")),
                                                         image_dict.get("model_detection_code"),
                                                         random_num,
                                                         image_dict.get("mode_service"),
                                                         self.msg.get('request_id'),  "OR")
                    ai_image_name = self.build_image_name(self.msg.get('results_base_dir'), time_now,
                                                          str(image_dict.get("current_frame")),
                                                          str(image_dict.get("last_frame")),
                                                          image_dict.get("model_detection_code"),
                                                          random_num,
                                                          image_dict.get("mode_service"),
                                                          self.msg.get('request_id'),  "AI")

                    # 上传原图片
                    aliyunOssSdk.upload_file(or_image_name, Image.fromarray(np.uint8(or_image)).tobytes())
                    aliyunOssSdk.upload_file(ai_image_name, Image.fromarray(np.uint8(ai_image)).tobytes())
                    # requestId, status, type, error_code="", error_msg="",  progress="", original_url="", sign_url="",
                    # category_id="", description="", analyse_time=""
                    # 发送kafka消息
                    self.sendResult(FeedBack.message_feedback(self.msg.get('request_id'), AnalysisStatus.RUNNING.value,
                                                              self.mode_service, "", "", image_dict.get("progress"),
                                                              or_image_name,
                                                              ai_image_name, image_dict.get("model_type_code"),
                                                              image_dict.get("model_detection_code"),
                                                              TimeUtils.now_date_to_str()))
            except Exception as e:
                logger.error("requestId:{}, 图片上传异常：", self.msg.get("request_id"))
                logger.exception(e)
        logger.info("结束图片上传线程, requestId:{}", self.msg.get("request_id"))

    def build_image_name(self, base_dir, time_now, current_frame, last_frame, descrition, random_num, mode_type,
                         requestId,  image_type):
        image_format = "{base_dir}/{time_now}_frame-{current_frame}-{last_frame}_type-{descrition}_{random_num}-{mode_type}-{base_dir}-{requestId}_{image_type}.jpg"
        image_name = image_format.format(
            base_dir=base_dir,
            time_now=time_now,
            current_frame=current_frame,
            last_frame=last_frame,
            descrition=descrition,
            random_num=random_num,
            mode_type=mode_type,
            requestId=requestId,
            image_type=image_type)
        return image_name
