# -*- coding: utf-8 -*-
from threading import Thread
import time
from loguru import logger
from util import TimeUtils
from enums.AnalysisStatusEnum import AnalysisStatus
from entity.FeedBack import message_feedback
from enums.AnalysisTypeEnum import AnalysisType


class Heartbeat(Thread):
    def __init__(self, fbQueue, hbQueue, request_id, mode_service):
        super().__init__()
        self.fbQueue = fbQueue
        self.hbQueue = hbQueue
        self.request_id = request_id
        self.mode_service = mode_service
        self.run_status = True
        self.progress = "0.0000"

    def getHbQueue(self):
        eBody = None
        if self.hbQueue is None:
            return eBody
        try:
            eBody = self.hbQueue.get(block=False)
        except Exception as e:
            pass
        return eBody

    # 推送执行结果
    def sendResult(self, result):
        self.fbQueue.put(result)

    def run(self):
        logger.info("开始启动心跳线程！requestId:{}", self.request_id)
        # 发送waiting状态信息
        feedback = message_feedback(self.request_id, AnalysisStatus.WAITING.value, self.mode_service,
                                    analyse_time=TimeUtils.now_date_to_str())
        self.sendResult(feedback)
        time.sleep(5)
        num = 0
        while self.run_status:
            hb_value = self.getHbQueue()
            if hb_value is not None:
                self.progress = hb_value
            if num % 60 == 0:
                if self.mode_service == AnalysisType.OFFLINE.value:
                    feedback = message_feedback(self.request_id, AnalysisStatus.RUNNING.value, self.mode_service,
                                                progress=self.progress,
                                                analyse_time=TimeUtils.now_date_to_str())
                if self.mode_service == AnalysisType.ONLINE.value:
                    feedback = message_feedback(self.request_id, AnalysisStatus.RUNNING.value, self.mode_service,
                                                analyse_time=TimeUtils.now_date_to_str())
                self.sendResult(feedback)
            num += 1
            time.sleep(1)
        logger.info("心跳线程停止完成！requestId:{}", self.request_id)
