# -*- coding: utf-8 -*-
import sys

sys.path.extend(['..', '../AIlib'])
from segutils.segmodel import SegModel, get_largest_contours
from utils.torch_utils import select_device
from utilsK.queRiver import get_labelnames, get_label_arrays, post_process_
from models.experimental import attempt_load
from AI import AI_process, AI_process_forest, get_postProcess_para


class Model():
    def __init__(self, device, allowedList=None):
        ##预先设置的参数
        self.device_ = device  ##选定模型，可选 cpu,'0','1'
        self.allowedList = allowedList


# 水面模型
class SZModel(Model):
    def __init__(self, device, allowedList=None):
        super().__init__(device, allowedList)
        ##以下参数目前不可改
        ##以下参数目前不可改
        #Detweights = "../AIlib/weights/yolov5/class5/best_5classes.pt"
        #labelnames = "../AIlib/weights/yolov5/class5/labelnames.json"
        Detweights = "../AIlib/weights/yolov5/class8/bestcao.pt" ##升级后的检测模型
        labelnames = "../AIlib/weights/yolov5/class8/labelnames.json" ##对应类别表

        seg_nclass = 2
        #Segweights = "../AIlib/weights/BiSeNet/checkpoint.pth"
        Segweights = '../AIlib/weights/STDC/model_maxmIOU75_1720_0.946_360640.pth' ##升级的分割模型
        postFile= '../AIlib/conf/para.json'
        self.conf_thres, self.iou_thres, self.classes, self.rainbows = get_postProcess_para(postFile)

        ##加载模型，准备好显示字符
        self.device = select_device(self.device_)
        self.names = get_labelnames(labelnames)
        self.label_arraylist = get_label_arrays(self.names, self.rainbows, outfontsize=40,
                                                fontpath="../AIlib/conf/platech.ttf")
        self.half = self.device.type != 'cpu'
        self.model = attempt_load(Detweights, map_location=self.device)
        if self.half:
            self.model.half()
        self.segmodel = SegModel(nclass=seg_nclass, weights=Segweights, device=self.device)

    def process(self, frame):
        return AI_process([frame], self.model, self.segmodel, self.names, self.label_arraylist,
                          self.rainbows, self.half, self.device, self.conf_thres, self.iou_thres,
                          self.allowedList)


# 林场模型
class LCModel(Model):
    def __init__(self, device, allowedList=None):
        super().__init__(device, allowedList)
        ##以下参数目前不可改
        Detweights = "../AIlib/weights/forest/best.pt"
        labelnames = "../AIlib/weights/forest/labelnames.json"
        postFile = '../AIlib/conf/para.json'

        self.conf_thres, self.iou_thres, self.classes, self.rainbows = get_postProcess_para(postFile)
        ##只加载检测模型，准备好显示字符
        self.device = select_device(self.device_)
        self.names = get_labelnames(labelnames)
        ###默认是1920，在森林巡检的高清图像中是4920
        imageW = 1920
        outfontsize = int(imageW/1920*40)
        self.label_arraylist = get_label_arrays(self.names, self.rainbows, outfontsize=outfontsize, fontpath="../AIlib/conf/platech.ttf")
        self.half = self.device.type != 'cpu'  # half precision only supported on CUDA
        self.model = attempt_load(Detweights, map_location=self.device)  # load FP32 model
        if self.half:
            self.model.half()
        self.segmodel = None

    def process(self, frame):
        return AI_process_forest([frame], self.model, self.segmodel, self.names, self.label_arraylist, self.rainbows,
                                 self.half, self.device, self.conf_thres, self.iou_thres, self.allowedList)
